@@ -2,6 +2,7 @@
 
 use strict;
 use Getopt::Long;
+use FB3;
 use FB3::Convert;
 use utf8;
 use File::ShareDir qw/dist_dir/;
@@ -33,7 +34,7 @@ GetOptions(
   'meta_date=s' => \$OPT{'meta_date'},
 ) || help();
 
-my $XsdPath = $OPT{xsd_dir} || dist_dir("FB3");
+my $XsdPath = $OPT{xsd_dir} || FB3::SchemasDirPath();
 my $XslPath = dist_dir('FB3-Convert');
 
 if ($OPT{'vl'}) {
